#-*- coding: utf-8 -*-
u"""
Component-based (X)HTML rendering.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2007
"""
from cocktail.html.element import Element, Content, PlaceHolder
from cocktail.html.resources import Resource, Script, StyleSheet
from cocktail.html.overlay import Overlay

